@@ -1,35 +1,24 @@
-# Copyright (c) 2016-2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2016-2019 Ashley Whetter <ashley@awhetter.co.uk>
-# Copyright (c) 2016 Yuri Bochkarev <baltazar.bz@gmail.com>
-# Copyright (c) 2016 Glenn Matthews <glenn@e-dad.net>
-# Copyright (c) 2016 Moises Lopez <moylop260@vauxoo.com>
-# Copyright (c) 2017, 2020 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2017 Mitar <mitar.github@tnode.com>
-# Copyright (c) 2018, 2020 Anthony Sottile <asottile@umich.edu>
-# Copyright (c) 2018 Jim Robertson <jrobertson98atx@gmail.com>
-# Copyright (c) 2018 ssolanki <sushobhitsolanki@gmail.com>
-# Copyright (c) 2018 Mitchell T.H. Young <mitchelly@gmail.com>
-# Copyright (c) 2018 Adrian Chirieac <chirieacam@gmail.com>
-# Copyright (c) 2019 Hugo van Kemenade <hugovk@users.noreply.github.com>
-# Copyright (c) 2019 Danny Hermes <daniel.j.hermes@gmail.com>
-# Copyright (c) 2019 Zeb Nicholls <zebedee.nicholls@climate-energy-college.org>
-# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-
 # Licensed under the GPL: https://www.gnu.org/licenses/old-licenses/gpl-2.0.html
-# For details: https://github.com/PyCQA/pylint/blob/master/COPYING
+# For details: https://github.com/pylint-dev/pylint/blob/main/LICENSE
+# Copyright (c) https://github.com/pylint-dev/pylint/blob/main/CONTRIBUTORS.txt
 
 """Utility methods for docstring checking."""
 
+from __future__ import annotations
+
+import itertools
 import re
-from typing import List
+from collections.abc import Iterable
 
 import astroid
+from astroid import nodes
+from astroid.util import UninferableBase
 
 from pylint.checkers import utils
 
 
-def space_indentation(s):
-    """The number of leading spaces in a string
+def space_indentation(s: str) -> int:
+    """The number of leading spaces in a string.
 
     :param str s: input string
 
@@ -39,7 +28,7 @@ def space_indentation(s):
     return len(s) - len(s.lstrip(" "))
 
 
-def get_setters_property_name(node):
+def get_setters_property_name(node: nodes.FunctionDef) -> str | None:
     """Get the name of the property that the given node is a setter for.
 
     :param node: The node to get the property name for.
@@ -52,21 +41,21 @@ def get_setters_property_name(node):
     decorators = node.decorators.nodes if node.decorators else []
     for decorator in decorators:
         if (
-            isinstance(decorator, astroid.Attribute)
+            isinstance(decorator, nodes.Attribute)
             and decorator.attrname == "setter"
-            and isinstance(decorator.expr, astroid.Name)
+            and isinstance(decorator.expr, nodes.Name)
         ):
-            return decorator.expr.name
+            return decorator.expr.name  # type: ignore[no-any-return]
     return None
 
 
-def get_setters_property(node):
+def get_setters_property(node: nodes.FunctionDef) -> nodes.FunctionDef | None:
     """Get the property node for the given setter node.
 
     :param node: The node to get the property for.
-    :type node: astroid.FunctionDef
+    :type node: nodes.FunctionDef
 
-    :rtype: astroid.FunctionDef or None
+    :rtype: nodes.FunctionDef or None
     :returns: The node relating to the property of the given setter node,
         or None if one could not be found.
     """
@@ -75,7 +64,7 @@ def get_setters_property(node):
     property_name = get_setters_property_name(node)
     class_node = utils.node_frame_class(node)
     if property_name and class_node:
-        class_attrs = class_node.getattr(node.name)
+        class_attrs: list[nodes.FunctionDef] = class_node.getattr(node.name)
         for attr in class_attrs:
             if utils.decorated_with_property(attr):
                 property_ = attr
@@ -84,7 +73,7 @@ def get_setters_property(node):
     return property_
 
 
-def returns_something(return_node):
+def returns_something(return_node: nodes.Return) -> bool:
     """Check if a return node returns a value other than None.
 
     :param return_node: The return node to check.
@@ -99,87 +88,195 @@ def returns_something(return_node):
     if returns is None:
         return False
 
-    return not (isinstance(returns, astroid.Const) and returns.value is None)
+    return not (isinstance(returns, nodes.Const) and returns.value is None)
 
 
-def _get_raise_target(node):
-    if isinstance(node.exc, astroid.Call):
+def _get_raise_target(node: nodes.NodeNG) -> nodes.NodeNG | UninferableBase | None:
+    if isinstance(node.exc, nodes.Call):
         func = node.exc.func
-        if isinstance(func, (astroid.Name, astroid.Attribute)):
+        if isinstance(func, (nodes.Name, nodes.Attribute)):
             return utils.safe_infer(func)
     return None
 
 
-def _split_multiple_exc_types(target: str) -> List[str]:
+def _split_multiple_exc_types(target: str) -> list[str]:
     delimiters = r"(\s*,(?:\s*or\s)?\s*|\s+or\s+)"
     return re.split(delimiters, target)
 
 
-def possible_exc_types(node):
-    """
-    Gets all of the possible raised exception types for the given raise node.
+def possible_exc_types(node: nodes.NodeNG) -> set[nodes.ClassDef]:
+    """Gets all the possible raised exception types for the given raise node.
 
     .. note::
 
         Caught exception types are ignored.
 
-
     :param node: The raise node to find exception types for.
-    :type node: astroid.node_classes.NodeNG
 
     :returns: A list of exception types possibly raised by :param:`node`.
-    :rtype: set(str)
     """
-    excs = []
-    if isinstance(node.exc, astroid.Name):
+    exceptions = []
+    if isinstance(node.exc, nodes.Name):
         inferred = utils.safe_infer(node.exc)
         if inferred:
-            excs = [inferred.name]
+            exceptions = [inferred]
     elif node.exc is None:
         handler = node.parent
-        while handler and not isinstance(handler, astroid.ExceptHandler):
+        while handler and not isinstance(handler, nodes.ExceptHandler):
             handler = handler.parent
 
         if handler and handler.type:
-            inferred_excs = astroid.unpack_infer(handler.type)
-            excs = (exc.name for exc in inferred_excs if exc is not astroid.Uninferable)
+            try:
+                for exception in astroid.unpack_infer(handler.type):
+                    if not isinstance(exception, UninferableBase):
+                        exceptions.append(exception)
+            except astroid.InferenceError:
+                pass
     else:
         target = _get_raise_target(node)
-        if isinstance(target, astroid.ClassDef):
-            excs = [target.name]
-        elif isinstance(target, astroid.FunctionDef):
-            for ret in target.nodes_of_class(astroid.Return):
+        if isinstance(target, nodes.ClassDef):
+            exceptions = [target]
+        elif isinstance(target, nodes.FunctionDef):
+            for ret in target.nodes_of_class(nodes.Return):
+                if ret.value is None:
+                    continue
                 if ret.frame() != target:
                     # return from inner function - ignore it
                     continue
 
                 val = utils.safe_infer(ret.value)
-                if (
-                    val
-                    and isinstance(val, (astroid.Instance, astroid.ClassDef))
-                    and utils.inherit_from_std_ex(val)
-                ):
-                    excs.append(val.name)
+                if val and utils.inherit_from_std_ex(val):
+                    if isinstance(val, nodes.ClassDef):
+                        exceptions.append(val)
+                    elif isinstance(val, astroid.Instance):
+                        exceptions.append(val.getattr("__class__")[0])
 
     try:
-        return {exc for exc in excs if not utils.node_ignores_exception(node, exc)}
+        return {
+            exc
+            for exc in exceptions
+            if not utils.node_ignores_exception(node, exc.name)
+        }
     except astroid.InferenceError:
         return set()
 
 
-def docstringify(docstring, default_type="default"):
-    for docstring_type in [
+def _is_ellipsis(node: nodes.NodeNG) -> bool:
+    return isinstance(node, nodes.Const) and node.value == Ellipsis
+
+
+def _merge_annotations(
+    annotations: Iterable[nodes.NodeNG], comment_annotations: Iterable[nodes.NodeNG]
+) -> Iterable[nodes.NodeNG | None]:
+    for ann, comment_ann in itertools.zip_longest(annotations, comment_annotations):
+        if ann and not _is_ellipsis(ann):
+            yield ann
+        elif comment_ann and not _is_ellipsis(comment_ann):
+            yield comment_ann
+        else:
+            yield None
+
+
+def _annotations_list(args_node: nodes.Arguments) -> list[nodes.NodeNG]:
+    """Get a merged list of annotations.
+
+    The annotations can come from:
+
+    * Real type annotations.
+    * A type comment on the function.
+    * A type common on the individual argument.
+
+    :param args_node: The node to get the annotations for.
+    :returns: The annotations.
+    """
+    plain_annotations = args_node.annotations or ()
+    func_comment_annotations = args_node.parent.type_comment_args or ()
+    comment_annotations = args_node.type_comment_posonlyargs
+    comment_annotations += args_node.type_comment_args or []
+    comment_annotations += args_node.type_comment_kwonlyargs
+    return list(
+        _merge_annotations(
+            plain_annotations,
+            _merge_annotations(func_comment_annotations, comment_annotations),
+        )
+    )
+
+
+def args_with_annotation(args_node: nodes.Arguments) -> set[str]:
+    result = set()
+    annotations = _annotations_list(args_node)
+    annotation_offset = 0
+
+    if args_node.posonlyargs:
+        posonlyargs_annotations = args_node.posonlyargs_annotations
+        if not any(args_node.posonlyargs_annotations):
+            num_args = len(args_node.posonlyargs)
+            posonlyargs_annotations = annotations[
+                annotation_offset : annotation_offset + num_args
+            ]
+            annotation_offset += num_args
+
+        for arg, annotation in zip(args_node.posonlyargs, posonlyargs_annotations):
+            if annotation:
+                result.add(arg.name)
+
+    if args_node.args:
+        num_args = len(args_node.args)
+        for arg, annotation in zip(
+            args_node.args,
+            annotations[annotation_offset : annotation_offset + num_args],
+        ):
+            if annotation:
+                result.add(arg.name)
+
+        annotation_offset += num_args
+
+    if args_node.vararg:
+        if args_node.varargannotation:
+            result.add(args_node.vararg)
+        elif len(annotations) > annotation_offset and annotations[annotation_offset]:
+            result.add(args_node.vararg)
+            annotation_offset += 1
+
+    if args_node.kwonlyargs:
+        kwonlyargs_annotations = args_node.kwonlyargs_annotations
+        if not any(args_node.kwonlyargs_annotations):
+            num_args = len(args_node.kwonlyargs)
+            kwonlyargs_annotations = annotations[
+                annotation_offset : annotation_offset + num_args
+            ]
+            annotation_offset += num_args
+
+        for arg, annotation in zip(args_node.kwonlyargs, kwonlyargs_annotations):
+            if annotation:
+                result.add(arg.name)
+
+    if args_node.kwarg:
+        if args_node.kwargannotation:
+            result.add(args_node.kwarg)
+        elif len(annotations) > annotation_offset and annotations[annotation_offset]:
+            result.add(args_node.kwarg)
+            annotation_offset += 1
+
+    return result
+
+
+def docstringify(
+    docstring: nodes.Const | None, default_type: str = "default"
+) -> Docstring:
+    best_match = (0, DOCSTRING_TYPES.get(default_type, Docstring)(docstring))
+    for docstring_type in (
         SphinxDocstring,
         EpytextDocstring,
         GoogleDocstring,
         NumpyDocstring,
-    ]:
+    ):
         instance = docstring_type(docstring)
-        if instance.is_valid():
-            return instance
+        matching_sections = instance.matching_sections()
+        if matching_sections > best_match[0]:
+            best_match = (matching_sections, instance)
 
-    docstring_type = DOCSTRING_TYPES.get(default_type, Docstring)
-    return docstring_type(docstring)
+    return best_match[1]
 
 
 class Docstring:
@@ -197,42 +294,45 @@ class Docstring:
     """
 
     # These methods are designed to be overridden
-    # pylint: disable=no-self-use
-    def __init__(self, doc):
-        doc = doc or ""
-        self.doc = doc.expandtabs()
+    def __init__(self, doc: nodes.Const | None) -> None:
+        docstring: str = doc.value if doc else ""
+        self.doc = docstring.expandtabs()
 
-    def is_valid(self):
-        return False
+    def __repr__(self) -> str:
+        return f"<{self.__class__.__name__}:'''{self.doc}'''>"
 
-    def exceptions(self):
+    def matching_sections(self) -> int:
+        """Returns the number of matching docstring sections."""
+        return 0
+
+    def exceptions(self) -> set[str]:
         return set()
 
-    def has_params(self):
+    def has_params(self) -> bool:
         return False
 
-    def has_returns(self):
+    def has_returns(self) -> bool:
         return False
 
-    def has_rtype(self):
+    def has_rtype(self) -> bool:
         return False
 
-    def has_property_returns(self):
+    def has_property_returns(self) -> bool:
         return False
 
-    def has_property_type(self):
+    def has_property_type(self) -> bool:
         return False
 
-    def has_yields(self):
+    def has_yields(self) -> bool:
         return False
 
-    def has_yields_type(self):
+    def has_yields_type(self) -> bool:
         return False
 
-    def match_param_docs(self):
+    def match_param_docs(self) -> tuple[set[str], set[str]]:
         return set(), set()
 
-    def params_documented_elsewhere(self):
+    def params_documented_elsewhere(self) -> bool:
         return self.re_for_parameters_see.search(self.doc) is not None
 
 
@@ -242,28 +342,22 @@ class SphinxDocstring(Docstring):
         \w(?:\w|\.[^\.])*    # Valid python name
         """
 
-    re_simple_container_type = r"""
-        {type}                        # a container type
+    re_simple_container_type = rf"""
+        {re_type}                     # a container type
         [\(\[] [^\n\s]+ [\)\]]        # with the contents of the container
-    """.format(
-        type=re_type
-    )
+    """
 
-    re_multiple_simple_type = r"""
-        (?:{container_type}|{type})
-        (?:(?:\s+(?:of|or)\s+|\s*,\s*)(?:{container_type}|{type}))*
-    """.format(
-        type=re_type, container_type=re_simple_container_type
-    )
+    re_multiple_simple_type = rf"""
+        (?:{re_simple_container_type}|{re_type})
+        (?:(?:\s+(?:of|or)\s+|\s*,\s*|\s+\|\s+)(?:{re_simple_container_type}|{re_type}))*
+    """
 
-    re_xref = r"""
+    re_xref = rf"""
         (?::\w+:)?                    # optional tag
-        `{}`                         # what to reference
-        """.format(
-        re_type
-    )
+        `{re_type}`                   # what to reference
+        """
 
-    re_param_raw = r"""
+    re_param_raw = rf"""
         :                       # initial colon
         (?:                     # Sphinx keywords
         param|parameter|
@@ -273,51 +367,43 @@ class SphinxDocstring(Docstring):
         \s+                     # whitespace
 
         (?:                     # optional type declaration
-        ({type}|{container_type})
+        ({re_type}|{re_simple_container_type})
         \s+
         )?
 
-        (\w+)                   # Parameter name
-        \s*                     # whitespace
-        :                       # final colon
-        """.format(
-        type=re_type, container_type=re_simple_container_type
-    )
+        ((\\\*{{0,2}}\w+)|(\w+))  # Parameter name with potential asterisks
+        \s*                       # whitespace
+        :                         # final colon
+        """
     re_param_in_docstring = re.compile(re_param_raw, re.X | re.S)
 
-    re_type_raw = r"""
-        :type                   # Sphinx keyword
-        \s+                     # whitespace
-        ({type})                # Parameter name
-        \s*                     # whitespace
-        :                       # final colon
-        """.format(
-        type=re_multiple_simple_type
-    )
+    re_type_raw = rf"""
+        :type                           # Sphinx keyword
+        \s+                             # whitespace
+        ({re_multiple_simple_type})     # Parameter name
+        \s*                             # whitespace
+        :                               # final colon
+        """
     re_type_in_docstring = re.compile(re_type_raw, re.X | re.S)
 
-    re_property_type_raw = r"""
-        :type:                  # Sphinx keyword
-        \s+                     # whitespace
-        {type}                  # type declaration
-        """.format(
-        type=re_multiple_simple_type
-    )
+    re_property_type_raw = rf"""
+        :type:                      # Sphinx keyword
+        \s+                         # whitespace
+        {re_multiple_simple_type}   # type declaration
+        """
     re_property_type_in_docstring = re.compile(re_property_type_raw, re.X | re.S)
 
-    re_raise_raw = r"""
-        :                       # initial colon
-        (?:                     # Sphinx keyword
+    re_raise_raw = rf"""
+        :                               # initial colon
+        (?:                             # Sphinx keyword
         raises?|
         except|exception
         )
-        \s+                     # whitespace
-        ({type})                # exception type
-        \s*                     # whitespace
-        :                       # final colon
-        """.format(
-        type=re_multiple_simple_type
-    )
+        \s+                             # whitespace
+        ({re_multiple_simple_type})     # exception type
+        \s*                             # whitespace
+        :                               # final colon
+        """
     re_raise_in_docstring = re.compile(re_raise_raw, re.X | re.S)
 
     re_rtype_in_docstring = re.compile(r":rtype:")
@@ -326,17 +412,21 @@ class SphinxDocstring(Docstring):
 
     supports_yields = False
 
-    def is_valid(self):
-        return bool(
-            self.re_param_in_docstring.search(self.doc)
-            or self.re_raise_in_docstring.search(self.doc)
-            or self.re_rtype_in_docstring.search(self.doc)
-            or self.re_returns_in_docstring.search(self.doc)
-            or self.re_property_type_in_docstring.search(self.doc)
+    def matching_sections(self) -> int:
+        """Returns the number of matching docstring sections."""
+        return sum(
+            bool(i)
+            for i in (
+                self.re_param_in_docstring.search(self.doc),
+                self.re_raise_in_docstring.search(self.doc),
+                self.re_rtype_in_docstring.search(self.doc),
+                self.re_returns_in_docstring.search(self.doc),
+                self.re_property_type_in_docstring.search(self.doc),
+            )
         )
 
-    def exceptions(self):
-        types = set()
+    def exceptions(self) -> set[str]:
+        types: set[str] = set()
 
         for match in re.finditer(self.re_raise_in_docstring, self.doc):
             raise_type = match.group(1)
@@ -344,25 +434,25 @@ class SphinxDocstring(Docstring):
 
         return types
 
-    def has_params(self):
+    def has_params(self) -> bool:
         if not self.doc:
             return False
 
         return self.re_param_in_docstring.search(self.doc) is not None
 
-    def has_returns(self):
+    def has_returns(self) -> bool:
         if not self.doc:
             return False
 
         return bool(self.re_returns_in_docstring.search(self.doc))
 
-    def has_rtype(self):
+    def has_rtype(self) -> bool:
         if not self.doc:
             return False
 
         return bool(self.re_rtype_in_docstring.search(self.doc))
 
-    def has_property_returns(self):
+    def has_property_returns(self) -> bool:
         if not self.doc:
             return False
 
@@ -370,18 +460,20 @@ class SphinxDocstring(Docstring):
         # so the first line must not be a known directive.
         return not self.doc.lstrip().startswith(":")
 
-    def has_property_type(self):
+    def has_property_type(self) -> bool:
         if not self.doc:
             return False
 
         return bool(self.re_property_type_in_docstring.search(self.doc))
 
-    def match_param_docs(self):
+    def match_param_docs(self) -> tuple[set[str], set[str]]:
         params_with_doc = set()
         params_with_type = set()
 
         for match in re.finditer(self.re_param_in_docstring, self.doc):
             name = match.group(2)
+            # Remove escape characters necessary for asterisks
+            name = name.replace("\\", "")
             params_with_doc.add(name)
             param_type = match.group(1)
             if param_type is not None:
@@ -392,8 +484,9 @@ class SphinxDocstring(Docstring):
 
 
 class EpytextDocstring(SphinxDocstring):
-    """
-    Epytext is similar to Sphinx. See the docs:
+    """Epytext is similar to Sphinx.
+
+    See the docs:
         http://epydoc.sourceforge.net/epytext.html
         http://epydoc.sourceforge.net/fields.html#fields
 
@@ -431,7 +524,7 @@ class EpytextDocstring(SphinxDocstring):
 
     re_returns_in_docstring = re.compile(r"@returns?:")
 
-    def has_property_returns(self):
+    def has_property_returns(self) -> bool:
         if not self.doc:
             return False
 
@@ -449,19 +542,15 @@ class GoogleDocstring(Docstring):
 
     re_xref = SphinxDocstring.re_xref
 
-    re_container_type = r"""
-        (?:{type}|{xref})             # a container type
+    re_container_type = rf"""
+        (?:{re_type}|{re_xref})       # a container type
         [\(\[] [^\n]+ [\)\]]          # with the contents of the container
-    """.format(
-        type=re_type, xref=re_xref
-    )
+    """
 
-    re_multiple_type = r"""
-        (?:{container_type}|{type}|{xref})
-        (?:(?:\s+(?:of|or)\s+|\s*,\s*)(?:{container_type}|{type}|{xref}))*
-    """.format(
-        type=re_type, xref=re_xref, container_type=re_container_type
-    )
+    re_multiple_type = rf"""
+        (?:{re_container_type}|{re_type}|{re_xref})
+        (?:(?:\s+(?:of|or)\s+|\s*,\s*|\s+\|\s+)(?:{re_container_type}|{re_type}|{re_xref}))*
+    """
 
     _re_section_template = r"""
         ^([ ]*)   {0} \s*:   \s*$     # Google parameter header
@@ -479,16 +568,14 @@ class GoogleDocstring(Docstring):
     )
 
     re_param_line = re.compile(
-        r"""
-        \s*  \*{{0,2}}(\w+)             # identifier potentially with asterisks
+        rf"""
+        \s*  ((?:\\?\*{{0,2}})?[\w\\]+) # identifier potentially with asterisks or escaped `\`
         \s*  ( [(]
-            {type}
+            {re_multiple_type}
             (?:,\s+optional)?
             [)] )? \s* :                # optional type declaration
         \s*  (.*)                       # beginning of optional description
-    """.format(
-            type=re_multiple_type
-        ),
+    """,
         re.X | re.S | re.M,
     )
 
@@ -497,12 +584,10 @@ class GoogleDocstring(Docstring):
     )
 
     re_raise_line = re.compile(
-        r"""
-        \s*  ({type}) \s* :              # identifier
+        rf"""
+        \s*  ({re_multiple_type}) \s* :  # identifier
         \s*  (.*)                        # beginning of optional description
-    """.format(
-            type=re_multiple_type
-        ),
+    """,
         re.X | re.S | re.M,
     )
 
@@ -511,22 +596,18 @@ class GoogleDocstring(Docstring):
     )
 
     re_returns_line = re.compile(
-        r"""
-        \s* ({type}:)?                    # identifier
+        rf"""
+        \s* ({re_multiple_type}:)?        # identifier
         \s* (.*)                          # beginning of description
-    """.format(
-            type=re_multiple_type
-        ),
+    """,
         re.X | re.S | re.M,
     )
 
     re_property_returns_line = re.compile(
-        r"""
-        ^{type}:                       # indentifier
+        rf"""
+        ^{re_multiple_type}:           # identifier
         \s* (.*)                       # Summary line / description
-    """.format(
-            type=re_multiple_type
-        ),
+    """,
         re.X | re.S | re.M,
     )
 
@@ -538,22 +619,26 @@ class GoogleDocstring(Docstring):
 
     supports_yields = True
 
-    def is_valid(self):
-        return bool(
-            self.re_param_section.search(self.doc)
-            or self.re_raise_section.search(self.doc)
-            or self.re_returns_section.search(self.doc)
-            or self.re_yields_section.search(self.doc)
-            or self.re_property_returns_line.search(self._first_line())
+    def matching_sections(self) -> int:
+        """Returns the number of matching docstring sections."""
+        return sum(
+            bool(i)
+            for i in (
+                self.re_param_section.search(self.doc),
+                self.re_raise_section.search(self.doc),
+                self.re_returns_section.search(self.doc),
+                self.re_yields_section.search(self.doc),
+                self.re_property_returns_line.search(self._first_line()),
+            )
         )
 
-    def has_params(self):
+    def has_params(self) -> bool:
         if not self.doc:
             return False
 
         return self.re_param_section.search(self.doc) is not None
 
-    def has_returns(self):
+    def has_returns(self) -> bool:
         if not self.doc:
             return False
 
@@ -569,7 +654,7 @@ class GoogleDocstring(Docstring):
 
         return False
 
-    def has_rtype(self):
+    def has_rtype(self) -> bool:
         if not self.doc:
             return False
 
@@ -585,7 +670,7 @@ class GoogleDocstring(Docstring):
 
         return False
 
-    def has_property_returns(self):
+    def has_property_returns(self) -> bool:
         # The summary line is the return doc,
         # so the first line must not be a known directive.
         first_line = self._first_line()
@@ -596,13 +681,13 @@ class GoogleDocstring(Docstring):
             or self.re_yields_section.search(first_line)
         )
 
-    def has_property_type(self):
+    def has_property_type(self) -> bool:
         if not self.doc:
             return False
 
         return bool(self.re_property_returns_line.match(self._first_line()))
 
-    def has_yields(self):
+    def has_yields(self) -> bool:
         if not self.doc:
             return False
 
@@ -618,7 +703,7 @@ class GoogleDocstring(Docstring):
 
         return False
 
-    def has_yields_type(self):
+    def has_yields_type(self) -> bool:
         if not self.doc:
             return False
 
@@ -634,8 +719,8 @@ class GoogleDocstring(Docstring):
 
         return False
 
-    def exceptions(self):
-        types = set()
+    def exceptions(self) -> set[str]:
+        types: set[str] = set()
 
         entries = self._parse_section(self.re_raise_section)
         for entry in entries:
@@ -650,9 +735,9 @@ class GoogleDocstring(Docstring):
 
         return types
 
-    def match_param_docs(self):
-        params_with_doc = set()
-        params_with_type = set()
+    def match_param_docs(self) -> tuple[set[str], set[str]]:
+        params_with_doc: set[str] = set()
+        params_with_type: set[str] = set()
 
         entries = self._parse_section(self.re_param_section)
         entries.extend(self._parse_section(self.re_keyword_param_section))
@@ -662,8 +747,12 @@ class GoogleDocstring(Docstring):
                 continue
 
             param_name = match.group(1)
+            # Remove escape characters necessary for asterisks
+            param_name = param_name.replace("\\", "")
+
             param_type = match.group(2)
             param_desc = match.group(3)
+
             if param_type:
                 params_with_type.add(param_name)
 
@@ -672,28 +761,28 @@ class GoogleDocstring(Docstring):
 
         return params_with_doc, params_with_type
 
-    def _first_line(self):
+    def _first_line(self) -> str:
         return self.doc.lstrip().split("\n", 1)[0]
 
     @staticmethod
-    def min_section_indent(section_match):
+    def min_section_indent(section_match: re.Match[str]) -> int:
         return len(section_match.group(1)) + 1
 
     @staticmethod
-    def _is_section_header(_):
+    def _is_section_header(_: str) -> bool:
         # Google parsing does not need to detect section headers,
         # because it works off of indentation level only
         return False
 
-    def _parse_section(self, section_re):
+    def _parse_section(self, section_re: re.Pattern[str]) -> list[str]:
         section_match = section_re.search(self.doc)
         if section_match is None:
             return []
 
         min_indentation = self.min_section_indent(section_match)
 
-        entries = []
-        entry = []
+        entries: list[str] = []
+        entry: list[str] = []
         is_first = True
         for line in section_match.group(2).splitlines():
             if not line.strip():
@@ -737,16 +826,26 @@ class NumpyDocstring(GoogleDocstring):
         re.X | re.S | re.M,
     )
 
+    re_default_value = r"""((['"]\w+\s*['"])|(\d+)|(True)|(False)|(None))"""
+
     re_param_line = re.compile(
-        r"""
-        \s*  (\w+)                      # identifier
-        \s*  :
-        \s*  (?:({type})(?:,\s+optional)?)? # optional type declaration
-        \n                              # description starts on a new line
-        \s* (.*)                        # description
-    """.format(
-            type=GoogleDocstring.re_multiple_type
-        ),
+        rf"""
+        \s*  (?P<param_name>\*{{0,2}}\w+)(\s?(:|\n)) # identifier with potential asterisks
+        \s*
+        (?P<param_type>
+         (
+          ({GoogleDocstring.re_multiple_type})      # default type declaration
+          (,\s+optional)?                           # optional 'optional' indication
+         )?
+         (
+          {{({re_default_value},?\s*)+}}            # set of default values
+         )?
+         (?:$|\n)
+        )?
+        (
+         \s* (?P<param_desc>.*)                     # optional description
+        )?
+    """,
         re.X | re.S,
     )
 
@@ -755,12 +854,10 @@ class NumpyDocstring(GoogleDocstring):
     )
 
     re_raise_line = re.compile(
-        r"""
-        \s* ({type})$   # type declaration
-        \s* (.*)        # optional description
-    """.format(
-            type=GoogleDocstring.re_type
-        ),
+        rf"""
+        \s* ({GoogleDocstring.re_type})$   # type declaration
+        \s* (.*)                           # optional description
+    """,
         re.X | re.S | re.M,
     )
 
@@ -769,13 +866,11 @@ class NumpyDocstring(GoogleDocstring):
     )
 
     re_returns_line = re.compile(
-        r"""
+        rf"""
         \s* (?:\w+\s+:\s+)? # optional name
-        ({type})$                         # type declaration
-        \s* (.*)                          # optional description
-    """.format(
-            type=GoogleDocstring.re_multiple_type
-        ),
+        ({GoogleDocstring.re_multiple_type})$   # type declaration
+        \s* (.*)                                # optional description
+    """,
         re.X | re.S | re.M,
     )
 
@@ -787,12 +882,54 @@ class NumpyDocstring(GoogleDocstring):
 
     supports_yields = True
 
+    def match_param_docs(self) -> tuple[set[str], set[str]]:
+        """Matches parameter documentation section to parameter documentation rules."""
+        params_with_doc = set()
+        params_with_type = set()
+
+        entries = self._parse_section(self.re_param_section)
+        entries.extend(self._parse_section(self.re_keyword_param_section))
+        for entry in entries:
+            match = self.re_param_line.match(entry)
+            if not match:
+                continue
+
+            # check if parameter has description only
+            re_only_desc = re.match(r"\s*(\*{0,2}\w+)\s*:?\n\s*\w*$", entry)
+            if re_only_desc:
+                param_name = match.group("param_name")
+                param_desc = match.group("param_type")
+                param_type = None
+            else:
+                param_name = match.group("param_name")
+                param_type = match.group("param_type")
+                param_desc = match.group("param_desc")
+                # The re_param_line pattern needs to match multi-line which removes the ability
+                # to match a single line description like 'arg : a number type.'
+                # We are not trying to determine whether 'a number type' is correct typing
+                # but we do accept it as typing as it is in the place where typing
+                # should be
+                if param_type is None and re.match(r"\s*(\*{0,2}\w+)\s*:.+$", entry):
+                    param_type = param_desc
+                # If the description is "" but we have a type description
+                # we consider the description to be the type
+                if not param_desc and param_type:
+                    param_desc = param_type
+
+            if param_type:
+                params_with_type.add(param_name)
+
+            if param_desc:
+                params_with_doc.add(param_name)
+
+        return params_with_doc, params_with_type
+
     @staticmethod
-    def min_section_indent(section_match):
+    def min_section_indent(section_match: re.Match[str]) -> int:
         return len(section_match.group(1))
 
     @staticmethod
-    def _is_section_header(line):
+    def _is_section_header(line: str) -> bool:
         return bool(re.match(r"\s*-+$", line))
 
 
